/**	Author: Travis Scott (ID 1554515)
 * 	Date: 14/12/2012
 */

package uoa.tsco033.evolisa;

import android.os.Bundle;
import android.app.ActionBar;
import android.app.ActionBar.Tab;
import android.app.Activity;
import android.app.Fragment;
import android.app.FragmentTransaction;
import android.view.Menu;

/**This activity allows the user to select the image to reproduce, and define the output settings
 * It uses a tab layout with two tabs, implemented using the fragment framework
 * 
 * @author tsco033
 */
public class ActivitySetup extends Activity
{
	private ActionBar actionBar;
	private ActionBar.Tab sourceImageTab;
	private ActionBar.Tab settingsTab;
	private Settings settings;

	//---------- Setup ----------//
	
	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_setup);

		// Initialize settings instance
		settings = new Settings(this.getSharedPreferences(Settings.SETTINGS_ID, 0));
		
		actionBar = getActionBar();
		actionBar.setNavigationMode(ActionBar.NAVIGATION_MODE_TABS);

		// Create tabs
		sourceImageTab = actionBar.newTab().setText(R.string.setup_tab_sourceimage);
		settingsTab = actionBar.newTab().setText(R.string.setup_tab_settings);

		// Attach listener to each tab
		sourceImageTab.setTabListener(new TabListener<FragmentSourceImage>(this, "sourceimage", FragmentSourceImage.class));
		settingsTab.setTabListener(new TabListener<FragmentSettings>(this, "settings", FragmentSettings.class));

		// Attach each tab to action bar
		actionBar.addTab(sourceImageTab);
		actionBar.addTab(settingsTab);
		
		// Check if we are restoring
		if (savedInstanceState != null) 
		{
			// If we are then return to the previously opened tab
			actionBar.setSelectedNavigationItem(savedInstanceState.getInt("tab"));
		}
	}
	
	//---------- Save state ----------//
	
	@Override
	protected void onSaveInstanceState(Bundle bundle)
	{
		super.onSaveInstanceState(bundle);
		
		// Store the current tab index
		bundle.putInt("tab", actionBar.getSelectedNavigationIndex());
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu)
	{
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.activity_home, menu);
		return true;
	}

	// ---------- Tab Listener ----------//
	
	private class TabListener<T extends Fragment> implements ActionBar.TabListener
	{
		private Activity parentActivity;
		private String fragmentTag;
		private Class<T> fragmentClass;

		public TabListener(Activity activity, String tag, Class<T> cls)
		{
			parentActivity = activity;
			fragmentTag = tag;
			fragmentClass = cls;
		}

		@Override
		public void onTabSelected(Tab tab, FragmentTransaction ft)
		{
			Fragment fragment = parentActivity.getFragmentManager().findFragmentByTag(fragmentTag);
			
			// Check if the fragment is already initialized
			if (fragment == null)
			{
				// If not, instantiate and add it to the activity
				fragment = Fragment.instantiate(parentActivity, fragmentClass.getName());
				ft.add(android.R.id.content, fragment, fragmentTag);
			}
			else
			{
				// If it exists, simply attach it in order to show it
				ft.attach(fragment);
			}
		}

		@Override
		public void onTabUnselected(Tab tab, FragmentTransaction ft)
		{
			Fragment fragment = parentActivity.getFragmentManager().findFragmentByTag(fragmentTag);
			if (fragment != null)
			{
				// Detach the fragment, because another one is being attached
				ft.detach(fragment);
			}
		}

		@Override
		public void onTabReselected(Tab tab, FragmentTransaction ft)
		{
		}
	}
	
}


