/**	Author: Travis Scott (ID 1554515)
 * 	Date: 14/12/2012
 */

package uoa.tsco033.evolisa;

import android.os.Parcel;
import android.os.Parcelable;

/**This class represents a colour and fill type, used to construct a polygon.
 * It stores the colour as an ARGB value
 * @author tsco033
 */
public class DNABrush implements Parcelable
{
	private int red;
	private int green;
	private int blue;
	private int alpha;

	// ---------- Setup ----------//

	public DNABrush()
	{
	}

	public DNABrush(int red, int green, int blue, int alpha)
	{
		this.red = red;
		this.green = green;
		this.blue = blue;
		this.alpha = alpha;
	}

	public DNABrush clone()
	{
		return new DNABrush(red, green, blue, alpha);
	}

	public void initialise()
	{
		red = Tools.getRandomNumber(Settings.ActiveRedRangeMin, Settings.ActiveRedRangeMax);
		green = Tools.getRandomNumber(Settings.ActiveGreenRangeMin, Settings.ActiveGreenRangeMax);
		blue = Tools.getRandomNumber(Settings.ActiveBlueRangeMin, Settings.ActiveBlueRangeMax);
		alpha = Tools.getRandomNumber(Settings.ActiveAlphaRangeMin, Settings.ActiveAlphaRangeMax);
	}

	// ---------- Parcelable Interface ----------//

	@Override
	public int describeContents()
	{
		return 0;
	}

	@Override
	public void writeToParcel(Parcel destination, int flags)
	{
		destination.writeInt(red);
		destination.writeInt(green);
		destination.writeInt(blue);
		destination.writeInt(alpha);
	}

	public DNABrush(Parcel source)
	{
		this.red = source.readInt();
		this.green = source.readInt();
		this.blue = source.readInt();
		this.alpha = source.readInt();
	}

	public static final Parcelable.Creator<DNABrush> DNABrushCreator = new Parcelable.Creator<DNABrush>()
	{
		public DNABrush createFromParcel(Parcel source)
		{
			return new DNABrush(source);
		}

		public DNABrush[] newArray(int size)
		{
			return new DNABrush[size];
		}
	};

	// ---------- Mutations ----------//

	/**Performs mutation of the brush
	 * @param drawing
	 */
	public void mutate(DNADrawing drawing)
	{
		// Check if red should be mutated
		if (Tools.willMutate(Settings.ActiveRedMutationRate))
		{
			red = Tools.getRandomNumber(Settings.ActiveRedRangeMin, Settings.ActiveRedRangeMax);
			drawing.setDirty();
		}
		// Check if green should be mutated
		if (Tools.willMutate(Settings.ActiveGreenMutationRate))
		{
			green = Tools.getRandomNumber(Settings.ActiveGreenRangeMin, Settings.ActiveGreenRangeMax);
			drawing.setDirty();
		}
		// Check if blue should be mutated
		if (Tools.willMutate(Settings.ActiveBlueMutationRate))
		{
			blue = Tools.getRandomNumber(Settings.ActiveBlueRangeMin, Settings.ActiveBlueRangeMax);
			drawing.setDirty();
		}
		// Check if alpha should be mutated
		if (Tools.willMutate(Settings.ActiveAlphaMutationRate))
		{
			alpha = Tools.getRandomNumber(Settings.ActiveAlphaRangeMin, Settings.ActiveAlphaRangeMax);
			drawing.setDirty();
		}
	}

	// ---------- Accessors ----------//

	/**Returns the brush colour as an int[] (A, R, G, B) 
	 * @return
	 */
	public int[] getColor()
	{
		int[] colours = { alpha, red, green, blue };
		return colours;
	}

}