/**	Author: Travis Scott (ID 1554515)
 * 	Date: 14/12/2012
 */

package uoa.tsco033.evolisa;

import android.os.Parcel;
import android.os.Parcelable;

/**This class represents a point, one of the building blocks of a polygon
 * It stores the x and y coordinates of the point 
 * @author tsco033
 */
public class DNAPoint implements Parcelable
{
	private int x;
	private int y;

	// ---------- Setup ----------//

	public DNAPoint()
	{
	}

	public DNAPoint(int x, int y)
	{
		this.x = x;
		this.y = y;
	}

	public DNAPoint clone()
	{
		return new DNAPoint(x, y);
	}

	public void initialise()
	{
		x = Tools.getRandomNumber(0, Settings.PictureWidth);
		y = Tools.getRandomNumber(0, Settings.PictureHeight);
	}

	// ---------- Parcelable Implementation ----------//

	@Override
	public int describeContents()
	{
		return 0;
	}

	@Override
	public void writeToParcel(Parcel destination, int flags)
	{
		// Store fields
		destination.writeInt(x);
		destination.writeInt(y);
	}

	public DNAPoint(Parcel source)
	{
		// Restore fields
		this.x = source.readInt();
		this.y = source.readInt();
	}

	public static final Parcelable.Creator<DNAPoint> DNAPointCreator = new Parcelable.Creator<DNAPoint>()
	{
		public DNAPoint createFromParcel(Parcel source)
		{
			return new DNAPoint(source);
		}

		public DNAPoint[] newArray(int size)
		{
			return new DNAPoint[size];
		}
	};

	// ---------- Mutations ----------//

	/**Performs mutation of the point
	 * @param drawing
	 */
	public void mutate(DNADrawing drawing)
	{
		// Check if point should be moved (full movement range)
		if (Tools.willMutate(Settings.ActiveMovePointMaxMutationRate))
		{
			x = Tools.getRandomNumber(0, Settings.PictureWidth);
			y = Tools.getRandomNumber(0, Settings.PictureHeight);
			// Flag the drawing as dirty
			drawing.setDirty();
		}
		// Check if point should be moved (shifts it by +/- MovePointRangeMid)
		if (Tools.willMutate(Settings.ActiveMovePointMidMutationRate))
		{
			x = Math.min(Math.max(0, x + Tools.getRandomNumber(-Settings.ActiveMovePointRangeMid, Settings.ActiveMovePointRangeMid)), Settings.PictureWidth);
			y = Math.min(Math.max(0, y + Tools.getRandomNumber(-Settings.ActiveMovePointRangeMid, Settings.ActiveMovePointRangeMid)), Settings.PictureHeight);
			// Flag the drawing as dirty
			drawing.setDirty();
		}
		// Check if point should be moved (shifts it by +/- MovePointRangeMin)
		if (Tools.willMutate(Settings.ActiveMovePointMinMutationRate))
		{
			x = Math.min(Math.max(0, x + Tools.getRandomNumber(-Settings.ActiveMovePointRangeMin, Settings.ActiveMovePointRangeMin)), Settings.PictureWidth);
			y = Math.min(Math.max(0, y + Tools.getRandomNumber(-Settings.ActiveMovePointRangeMin, Settings.ActiveMovePointRangeMin)), Settings.PictureHeight);
			// Flag the drawing as dirty
			drawing.setDirty();
		}
	}

	// ---------- Accessors ----------//

	/** Returns the y coordinate
	 * @return
	 */
	public int getY()
	{
		return y;
	}

	/**Returns the x coordinate 
	 * @return
	 */
	public int getX()
	{
		return x;
	}

}