/**	Author: Travis Scott (ID 1554515)
 * 	Date: 14/12/2012
 */

package uoa.tsco033.evolisa;

import java.nio.BufferUnderflowException;
import java.nio.ByteBuffer;

import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.util.Log;

/**This class calculates how closely a generated image matches the source image.
 * It does this by taking the sum of the squares of the differences in colour values
 * for each pixel.
 * @author tsco033
 */
public class FitnessCalculator
{
	private Bitmap bitmap;
	private ByteBuffer buffer;
	private Canvas canvas;
	private int rows;
	private int columns;
	private double error;

	//---------- Setup ----------//
	
	public FitnessCalculator()
	{
		// Create a new bitmap object
		bitmap = Bitmap.createBitmap(Settings.PictureWidth, Settings.PictureHeight, Bitmap.Config.ARGB_8888);
		
		// Create a new canvas, drawing onto the bitmap
		canvas = new Canvas(bitmap);
		
		// Create a buffer to hold generated picture pixel values (much faster than calling getPixel on Bitmap)
		buffer = ByteBuffer.allocate(bitmap.getWidth() * bitmap.getHeight() * 4);
		
		// Set number of rows and columns
		rows = Settings.PictureHeight;
		columns = Settings.PictureWidth * 3;
	}
	
	//---------- Methods ----------//

	/**Returns the fitness (correlation) for a given drawing and source colour array
	 * @param drawing
	 * @param sourceColorsArray
	 * @return
	 */
	public double getDrawingFitness(DNADrawing drawing, byte[][] sourceColorsArray)
	{
		// Reset error to zero
		error = 0;

		// Paint the polygons to a canvas object (sets pixels)
		Renderer.render(drawing, canvas, 1);

		// Return to start of buffer
		buffer.rewind();

		// Copy bitmap pixel values into buffer
		bitmap.copyPixelsToBuffer(buffer);

		// Return to start of buffer
		buffer.rewind();

		// Loop through each row
		for (int row = 0; row < rows; row++)
		{
			// Loop through each column, incrementing by 3 as each pixel has RGB values, so takes 3 bytes
			for (int column = 0; column < columns; column += 3)
			{
				// Extract the pixel value and subtract the source value to get differences
				int r = (buffer.get() & 0xFF) - (sourceColorsArray[row][column + 0] & 0xFF);
				int g = (buffer.get() & 0xFF) - (sourceColorsArray[row][column + 1] & 0xFF);
				int b = (buffer.get() & 0xFF) - (sourceColorsArray[row][column + 2] & 0xFF);
				buffer.get(); // Alpha value ignored

				// Calculate the error by summing the squares
				error += r * r + g * g + b * b;
			}
		}

		return error;
	}
}
