/**	Author: Travis Scott (ID 1554515)
 * 	Date: 14/12/2012
 */

package uoa.tsco033.evolisa;

import android.content.SharedPreferences;
import android.os.Bundle;
import android.preference.PreferenceFragment;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;

/**This class is used to display the image settings.
*  It is implemented as a fragment, and allows users to change settings using drop down boxes
*  @author tsco033
*/
public class FragmentSettings extends PreferenceFragment
{
	private SharedPreferences settingsPreferences;
	private SharedPreferences.Editor settingsPreferencesEditor;

	private Button resetButton;
	
	private SettingEntry maxPictureHeight;
	private SettingEntry maxPictureWidth;
	private SettingEntry polygonsMax;
	private SettingEntry polygonsMin;
	private SettingEntry pointsPerPolygonMax;
	private SettingEntry pointsPerPolygonMin;
	private SettingEntry pointsMax;
	private SettingEntry pointsMin;
	private SettingEntry addPolygonMutationRate;
	private SettingEntry movePolygonMutationRate;
	private SettingEntry removePolygonMutationRate;
	private SettingEntry addPointMutationRate;
	private SettingEntry removePointMutationRate;
	private SettingEntry movePointMaxMutationRate;
	private SettingEntry movePointMidMutationRate;
	private SettingEntry movePointMinMutationRate;
	private SettingEntry movePointRangeMid;
	private SettingEntry movePointRangeMin;
	private SettingEntry alphaMutationRate;
	private SettingEntry alphaRangeMax;
	private SettingEntry alphaRangeMin;
	private SettingEntry blueMutationRate;
	private SettingEntry greenMutationRate;
	private SettingEntry redMutationRate;

	//--------- Setup ----------//
	
	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState)
	{
		// Get access to the application shared preferences
		settingsPreferences = this.getActivity().getSharedPreferences(Settings.SETTINGS_ID, 0);
		settingsPreferencesEditor = settingsPreferences.edit();

		// Inflate the layout view from file
		View view = inflater.inflate(R.layout.tab_settings, container, false);
		
		// Create the settings objects
		maxPictureHeight = new SettingEntry((EditText) view.findViewById(R.id.setup_settings_edittext_maxpictureheight), "maxpictureheight", Settings.DefaultMaxHeight);
		maxPictureWidth = new SettingEntry((EditText) view.findViewById(R.id.setup_settings_edittext_maxpicturewidth), "maxpicturewidth", Settings.DefaultMaxWidth);
		polygonsMax = new SettingEntry((EditText) view.findViewById(R.id.setup_settings_edittext_polygonsmax), "polygonsmax", Settings.DefaultPolygonsMax);
		polygonsMin = new SettingEntry((EditText) view.findViewById(R.id.setup_settings_edittext_polygonsmin), "polygonsmin", Settings.DefaultPolygonsMin);
		pointsPerPolygonMax = new SettingEntry((EditText) view.findViewById(R.id.setup_settings_edittext_pointsperpolygonmax), "pointsperpolygonmax", Settings.DefaultPointsPerPolygonMax);
		pointsPerPolygonMin = new SettingEntry((EditText) view.findViewById(R.id.setup_settings_edittext_pointsperpolygonmin), "pointsperpolygonmin", Settings.DefaultPointsPerPolygonMin);
		pointsMax = new SettingEntry((EditText) view.findViewById(R.id.setup_settings_edittext_pointsmax), "pointsmax", Settings.DefaultPointsMax);
		pointsMin = new SettingEntry((EditText) view.findViewById(R.id.setup_settings_edittext_pointsmin), "pointsmin", Settings.DefaultPointsMin);
		addPolygonMutationRate = new SettingEntry((EditText) view.findViewById(R.id.setup_settings_edittext_addpolygonmutationrate), "addpolygonmutationrate", Settings.DefaultAddPolygonMutationRate);
		movePolygonMutationRate = new SettingEntry((EditText) view.findViewById(R.id.setup_settings_edittext_movepolygonmutationrate), "movepolygonmutationrate", Settings.DefaultMovePolygonMutationRate);
		removePolygonMutationRate = new SettingEntry((EditText) view.findViewById(R.id.setup_settings_edittext_removepolygonmutationrate), "removepolygonmutationrate", Settings.DefaultRemovePolygonMutationRate);
		addPointMutationRate = new SettingEntry((EditText) view.findViewById(R.id.setup_settings_edittext_addpointmutationrate), "addpointmutationrate", Settings.DefaultAddPointMutationRate);
		removePointMutationRate = new SettingEntry((EditText) view.findViewById(R.id.setup_settings_edittext_removepointmutationrate), "removepointmutationrate", Settings.DefaultRemovePointMutationRate);
		movePointMaxMutationRate = new SettingEntry((EditText) view.findViewById(R.id.setup_settings_edittext_movepointmaxmutationrate), "movepointmaxmutationrate", Settings.DefaultMovePointMaxMutationRate);
		movePointMidMutationRate = new SettingEntry((EditText) view.findViewById(R.id.setup_settings_edittext_movepointmidmutationrate), "movepointmidmutationrate", Settings.DefaultMovePointMidMutationRate);
		movePointMinMutationRate = new SettingEntry((EditText) view.findViewById(R.id.setup_settings_edittext_movepointminmutationrate), "movepointminmutationrate", Settings.DefaultMovePointMinMutationRate);
		movePointRangeMid = new SettingEntry((EditText) view.findViewById(R.id.setup_settings_edittext_movepointrangemid), "movepointrangemid", Settings.DefaultMovePointRangeMid);
		movePointRangeMin = new SettingEntry((EditText) view.findViewById(R.id.setup_settings_edittext_movepointrangemin), "movepointrangemin", Settings.DefaultMovePointRangeMin);
		alphaMutationRate = new SettingEntry((EditText) view.findViewById(R.id.setup_settings_edittext_alphamutationrate), "alphamutationrate", Settings.DefaultAlphaMutationRate);
		alphaRangeMax = new SettingEntry((EditText) view.findViewById(R.id.setup_settings_edittext_alpharangemax), "alpharangemax", Settings.DefaultAlphaRangeMax);
		alphaRangeMin = new SettingEntry((EditText) view.findViewById(R.id.setup_settings_edittext_alpharangemin), "alpharangemin", Settings.DefaultAlphaRangeMin);
		blueMutationRate = new SettingEntry((EditText) view.findViewById(R.id.setup_settings_edittext_bluemutationrate), "bluemutationrate", Settings.DefaultBlueMutationRate);
		greenMutationRate = new SettingEntry((EditText) view.findViewById(R.id.setup_settings_edittext_greenmutationrate), "greenmutationrate", Settings.DefaultGreenMutationRate);
		redMutationRate = new SettingEntry((EditText) view.findViewById(R.id.setup_settings_edittext_redmutationrate), "redmutationrate", Settings.DefaultRedMutationRate);
		
		// Setup handle to widgets from the inflated view
		resetButton = (Button) view.findViewById(R.id.setup_settings_button_reset);
		
		// Set the values for the settings objects
		maxPictureHeight.setValue();
		maxPictureWidth.setValue();
		polygonsMax.setValue();
		polygonsMin.setValue();
		pointsPerPolygonMax.setValue();
		pointsPerPolygonMin.setValue();
		pointsMax.setValue();
		pointsMin.setValue();
		addPolygonMutationRate.setValue();
		movePolygonMutationRate.setValue();
		removePolygonMutationRate.setValue();
		addPointMutationRate.setValue();
		removePointMutationRate.setValue();
		movePointMaxMutationRate.setValue();
		movePointMidMutationRate.setValue();
		movePointMinMutationRate.setValue();
		movePointRangeMid.setValue();
		movePointRangeMin.setValue();
		alphaMutationRate.setValue();
		alphaRangeMax.setValue();
		alphaRangeMin.setValue();
		blueMutationRate.setValue();
		greenMutationRate.setValue();
		redMutationRate.setValue();

		setupButtons();

		return view;
	}

	//--------- Methods ----------//
	
	private void setupButtons()
	{
		resetButton.setOnClickListener(new View.OnClickListener()
		{			
			@Override
			public void onClick(View v)
			{
				maxPictureHeight.setDefault();
				maxPictureWidth.setDefault();
				polygonsMax.setDefault();
				polygonsMin.setDefault();
				pointsPerPolygonMax.setDefault();
				pointsPerPolygonMin.setDefault();
				pointsMax.setDefault();
				pointsMin.setDefault();
				addPolygonMutationRate.setDefault();
				movePolygonMutationRate.setDefault();
				removePolygonMutationRate.setDefault();
				addPointMutationRate.setDefault();
				removePointMutationRate.setDefault();
				movePointMaxMutationRate.setDefault();
				movePointMidMutationRate.setDefault();
				movePointMinMutationRate.setDefault();
				movePointRangeMid.setDefault();
				movePointRangeMin.setDefault();
				alphaMutationRate.setDefault();
				alphaRangeMax.setDefault();
				alphaRangeMin.setDefault();
				blueMutationRate.setDefault();
				greenMutationRate.setDefault();
				redMutationRate.setDefault();
			}
		});
	}

	/**This class wraps the data required for each setting entry.
	 * It is initialized with the widget, parameter name, and default value.
	 * It then initializes listeners for the widget, and ensures changes are
	 * stored back to the application shared preferences
	 * @author tsco033	 *
	 */
	private class SettingEntry implements TextWatcher
	{
		private EditText editText;
		private String name;
		private int defaultValue;

		//--------- Setup ----------//
		
		public SettingEntry(EditText editText, String name, int defaultValue)
		{
			this.editText = editText;
			this.name = name;
			this.defaultValue = defaultValue;
			this.editText.addTextChangedListener(this);
		}

		//--------- Methods ----------//
		
		/** Set the parameter to the value currently stored in share preferences **/
		public void setValue() 
		{
			editText.setText(Integer.toString(settingsPreferences.getInt(name, defaultValue)));
		}				
		
		/** Reset the parameter to default value **/
		public void setDefault() 
		{
			editText.setText(Integer.toString(defaultValue));
		}
		
		//--------- Interface Requirements ----------//
		
		@Override
		public void onTextChanged(CharSequence s, int start, int before, int count)
		{
			// Check if the value has been cleared
			if (s.length() == 0) 
			{
				// Store 0 to the shared preferences
				settingsPreferencesEditor.putInt(name, 0);
				settingsPreferencesEditor.commit();
				editText.setText(Integer.toString(0));
			}
			// Check if the value has been changed
			else if (s != "")
			{
				// Store the new value to the shared preferences
				settingsPreferencesEditor.putInt(name, Integer.parseInt(editText.getText().toString()));
				settingsPreferencesEditor.commit();
			}
			
			// Update the active values
			Settings.updateSettings(settingsPreferences);
		}

		@Override
		public void afterTextChanged(Editable s) {}

		@Override
		public void beforeTextChanged(CharSequence s, int start, int count, int after) {}
	}
		
}
