/**	Author: Travis Scott (ID 1554515)
 * 	Date: 14/12/2012
 */

package uoa.tsco033.evolisa;

import android.content.SharedPreferences;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;

/** This class contains settings used to control what objects are used to generate the output image 
 *  @author tsco033
 */
public class Settings
{
	public static final String SETTINGS_ID = "EvoLisa_Settings";
	
	// Set at runtime by ActivityGenerateImage
	
	public static int PictureHeight;
	public static int PictureWidth;
	public static int PolygonsPerThreadMax;
	public static int PolygonsPerThreadMin;
	
	// Current Settings
	
	public static int ActiveMaxWidth;
	public static int ActiveMaxHeight;
	public static int ActivePolygonsMax;
	public static int ActivePolygonsMin;
	public static int ActivePointsPerPolygonMax;
	public static int ActivePointsPerPolygonMin;
	public static int ActivePointsMax;
	public static int ActivePointsMin;
	public static int ActiveAddPolygonMutationRate;
	public static int ActiveMovePolygonMutationRate;
	public static int ActiveRemovePolygonMutationRate;
	public static int ActiveAddPointMutationRate;
	public static int ActiveRemovePointMutationRate;
	public static int ActiveMovePointMaxMutationRate;
	public static int ActiveMovePointMidMutationRate;
	public static int ActiveMovePointMinMutationRate;
	public static int ActiveMovePointRangeMid;
	public static int ActiveMovePointRangeMin;
	public static int ActiveAlphaMutationRate;
	public static int ActiveAlphaRangeMax;
	public static int ActiveAlphaRangeMin;
	public static int ActiveBlueMutationRate;
	public static int ActiveBlueRangeMax;
	public static int ActiveBlueRangeMin;
	public static int ActiveGreenMutationRate;
	public static int ActiveGreenRangeMax;
	public static int ActiveGreenRangeMin;
	public static int ActiveRedMutationRate;
	public static int ActiveRedRangeMax;
	public static int ActiveRedRangeMin;

	// Default Settings
	
	public static final int DefaultMaxWidth = 200;
	public static final int DefaultMaxHeight = 200;
	public static final int DefaultPolygonsMax = 50;
	public static final int DefaultPolygonsMin = 20;
	public static final int DefaultPointsPerPolygonMax = 10;
	public static final int DefaultPointsPerPolygonMin = 3;
	public static final int DefaultPointsMax = 1500;
	public static final int DefaultPointsMin = 0;
	public static final int DefaultAddPolygonMutationRate = 700;
	public static final int DefaultMovePolygonMutationRate = 700;
	public static final int DefaultRemovePolygonMutationRate = 1500;
	public static final int DefaultAddPointMutationRate = 1500;
	public static final int DefaultRemovePointMutationRate = 1500;
	public static final int DefaultMovePointMaxMutationRate = 1500;
	public static final int DefaultMovePointMidMutationRate = 1500;
	public static final int DefaultMovePointMinMutationRate = 1500;
	public static final int DefaultMovePointRangeMid = 20;
	public static final int DefaultMovePointRangeMin = 3;
	public static final int DefaultAlphaMutationRate = 1500;
	public static final int DefaultAlphaRangeMax = 60;
	public static final int DefaultAlphaRangeMin = 30;
	public static final int DefaultBlueMutationRate = 1500;
	public static final int DefaultBlueRangeMax = 255;
	public static final int DefaultBlueRangeMin = 0;
	public static final int DefaultGreenMutationRate = 1500;
	public static final int DefaultGreenRangeMax = 255;
	public static final int DefaultGreenRangeMin = 0;
	public static final int DefaultRedMutationRate = 1500;
	public static final int DefaultRedRangeMax = 255;
	public static final int DefaultRedRangeMin = 0;
	
	/**Constructor requires a handle to the application Shared Preferences	 * 
	 * @param settingsPreferences
	 */
	public Settings(SharedPreferences settingsPreferences)
	{		
		updateSettings(settingsPreferences);
		Settings.ActiveBlueRangeMax = 255;
		Settings.ActiveBlueRangeMin = 0;
		Settings.ActiveGreenRangeMax = 255;
		Settings.ActiveGreenRangeMin = 0;
		Settings.ActiveRedRangeMax = 255;
		Settings.ActiveRedRangeMin = 0;
	}
	
	/**This method updates the settings, given the application shared preferences	 * 
	 * @param settingsPreferences The application shared preferences
	 */
	public static void updateSettings(SharedPreferences settingsPreferences)
	{		
		ActiveMaxHeight = settingsPreferences.getInt("maxpictureheight", Settings.DefaultMaxHeight);
		ActiveMaxWidth = settingsPreferences.getInt("maxpicturewidth", Settings.DefaultMaxWidth);
		ActivePolygonsMax = settingsPreferences.getInt("polygonsmax", Settings.DefaultPolygonsMax);
		ActivePolygonsMin = settingsPreferences.getInt("polygonsmin", Settings.DefaultPolygonsMin);
		ActivePointsPerPolygonMax = settingsPreferences.getInt("pointsperpolygonmax", Settings.DefaultPointsPerPolygonMax);
		ActivePointsPerPolygonMin = settingsPreferences.getInt("pointsperpolygonmin", Settings.DefaultPointsPerPolygonMin);
		ActivePointsMax = settingsPreferences.getInt("pointsmax", Settings.DefaultPointsMax);
		ActivePointsMin = settingsPreferences.getInt("pointsmin", Settings.DefaultPointsMin);
		ActiveAddPolygonMutationRate = settingsPreferences.getInt("addpolygonmutationrate", Settings.DefaultAddPolygonMutationRate);
		ActiveMovePolygonMutationRate = settingsPreferences.getInt("movepolygonmutationrate", Settings.DefaultMovePolygonMutationRate);
		ActiveRemovePolygonMutationRate = settingsPreferences.getInt("removepolygonmutationrate", Settings.DefaultRemovePolygonMutationRate);
		ActiveAddPointMutationRate = settingsPreferences.getInt("addpointmutationrate", Settings.DefaultAddPointMutationRate);
		ActiveRemovePointMutationRate = settingsPreferences.getInt("removepointmutationrate", Settings.DefaultRemovePointMutationRate);
		ActiveMovePointMaxMutationRate = settingsPreferences.getInt("movepointmaxmutationrate", Settings.DefaultMovePointMaxMutationRate);
		ActiveMovePointMidMutationRate = settingsPreferences.getInt("movepointmidmutationrate", Settings.DefaultMovePointMidMutationRate);
		ActiveMovePointMinMutationRate = settingsPreferences.getInt("movepointminmutationrate", Settings.DefaultMovePointMinMutationRate);
		ActiveMovePointRangeMid = settingsPreferences.getInt("movepointrangemid", Settings.DefaultMovePointRangeMid);
		ActiveMovePointRangeMin = settingsPreferences.getInt("movepointrangemin", Settings.DefaultMovePointRangeMin);
		ActiveAlphaMutationRate = settingsPreferences.getInt("alphamutationrate", Settings.DefaultAlphaMutationRate);
		ActiveAlphaRangeMax = settingsPreferences.getInt("alpharangemax", Settings.DefaultAlphaRangeMax);
		ActiveAlphaRangeMin = settingsPreferences.getInt("alpharangemin", Settings.DefaultAlphaRangeMin);
		ActiveBlueMutationRate = settingsPreferences.getInt("bluemutationrate", Settings.DefaultBlueMutationRate);
		ActiveGreenMutationRate = settingsPreferences.getInt("greenmutationrate", Settings.DefaultGreenMutationRate);
		ActiveRedMutationRate = settingsPreferences.getInt("redmutationrate", Settings.DefaultRedMutationRate);	
	}	
	
	/*
	public static void updateSettings(
		int ActiveMaxWidth,
		int ActiveMaxHeight,
		int ActivePolygonsMax,
		int ActivePolygonsMin,
		int ActivePointsPerPolygonMax,
		int ActivePointsPerPolygonMin,
		int ActivePointsMax,
		int ActivePointsMin,
		int ActiveAddPolygonMutationRate,
		int ActiveMovePolygonMutationRate,
		int ActiveRemovePolygonMutationRate,
		int ActiveAddPointMutationRate,
		int ActiveRemovePointMutationRate,
		int ActiveMovePointMaxMutationRate,
		int ActiveMovePointMidMutationRate,
		int ActiveMovePointMinMutationRate,
		int ActiveMovePointRangeMid,
		int ActiveMovePointRangeMin,
		int ActiveAlphaMutationRate,
		int ActiveAlphaRangeMax,
		int ActiveAlphaRangeMin,
		int ActiveBlueMutationRate,
		int ActiveGreenMutationRate,
		int ActiveRedMutationRate)
	{
		Settings.ActiveMaxWidth = ActiveMaxWidth;
		Settings.ActiveMaxHeight = ActiveMaxHeight;
		Settings.ActivePolygonsMax = ActivePolygonsMax;
		Settings.ActivePolygonsMin = ActivePolygonsMin;
		Settings.ActivePointsPerPolygonMax = ActivePointsPerPolygonMax;
		Settings.ActivePointsPerPolygonMin = ActivePointsPerPolygonMin;
		Settings.ActivePointsMax = ActivePointsMax;
		Settings.ActivePointsMin = ActivePointsMin;
		Settings.ActiveAddPolygonMutationRate = ActiveAddPolygonMutationRate;
		Settings.ActiveMovePolygonMutationRate = ActiveMovePolygonMutationRate;
		Settings.ActiveRemovePolygonMutationRate = ActiveRemovePolygonMutationRate;
		Settings.ActiveAddPointMutationRate = ActiveAddPointMutationRate;
		Settings.ActiveRemovePointMutationRate = ActiveRemovePointMutationRate;
		Settings.ActiveMovePointMaxMutationRate = ActiveMovePointMaxMutationRate;
		Settings.ActiveMovePointMidMutationRate = ActiveMovePointMidMutationRate;
		Settings.ActiveMovePointMinMutationRate = ActiveMovePointMinMutationRate;
		Settings.ActiveMovePointRangeMid = ActiveMovePointRangeMid;
		Settings.ActiveMovePointRangeMin = ActiveMovePointRangeMin;
		Settings.ActiveAlphaMutationRate = ActiveAlphaMutationRate;
		Settings.ActiveAlphaRangeMax = ActiveAlphaRangeMax;
		Settings.ActiveAlphaRangeMin = ActiveAlphaRangeMin;
		Settings.ActiveBlueMutationRate = ActiveBlueMutationRate;
		Settings.ActiveGreenMutationRate = ActiveGreenMutationRate;
		Settings.ActiveRedMutationRate = ActiveRedMutationRate;
		Settings.ActiveBlueRangeMax = 255;
		Settings.ActiveBlueRangeMin = 0;
		Settings.ActiveGreenRangeMax = 255;
		Settings.ActiveGreenRangeMin = 0;
		Settings.ActiveRedRangeMax = 255;
		Settings.ActiveRedRangeMin = 0;
	}*/
}